//Copyright 2022 JerryLive
//Handles a Flyer (except Player).

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class AIScript : MonoBehaviour
{

//Forward for a Flyer is the +X axis, transform.right.
//The maximum Force and Torque sent to FlyerScript MUST match the Player.
	private const float MAX_FORCE = 1f;
	private const float LOW_FORCE = 0.3f;
	private const float MIN_FORCE = 0f;
	private const float MAX_TORQUE = 1f;
	private const float MIN_TORQUE = -1f;

//A Target Flyer is set by FlyerMaster for every AI Flyer.
	private int targetIndex;
	private float distanceToTarget;
	private Vector3 headingToTarget;
	private Vector3 myForward;

//Current values.
	private Vector3 desiredHeading;
	private float desiredForce;
	private float desiredTorque;

//Missles.
//Missles can not be fired any faster than the Player.
	private int missleFrameCount;
	private bool missleBusy;

//References.
	public FlyerScript FS;


//Start here.#####################################################
	private void Awake()
	{
		missleBusy = false;
		desiredForce = 0f;
		desiredTorque = 0f;
		//State = enumState.ATTACK;
	}

//Every Physics Frame.
	private void FixedUpdate()
	{
		if (Global.DoNotRun)
			return;
	//Everything about the Target.
		targetIndex = Global.Flyer[FS.myIndex].targetIndex;
		distanceToTarget = Global.Where[FS.myIndex, targetIndex].distance;
		headingToTarget = Global.Where[FS.myIndex, targetIndex].heading;
		myForward = Global.Flyer[FS.myIndex].GO.transform.right;
	//Decide what to do...
		Decide();
	//And do it.
		FS.FlyerForce = desiredForce;
		FS.FlyerTorque = desiredTorque;
	//Missle.
		if (missleBusy)
		{
			if (missleFrameCount <= 0)
				missleBusy = false;
			else
				missleFrameCount -= 1;
		}
	}

//Fire Missle.
	private void FireMissle()
	{
		if (missleBusy)
			return;
		FS.FireMissle();
		missleBusy = true;
		missleFrameCount = Global.AI_FramesPerMissle;
	}


//Decision.#######################################################
//The goal of an AI Flyer is to kill the Target.
//Firing Missles or colliding both work.

//Distances.
	private const float FIRE_DISTANCE = Global.MISSLE_KILL_DISTANCE;
	private const float STOP_FIRE_DISTANCE = 1.25f;
	private const float RETREAT_DISTANCE = 0.7f * FIRE_DISTANCE;

//The strategy depends on the distance between the Flyer and the Target.
//There are 3 zones:
//Farther than FIRE_DISTANCE.
	//Head at Target, max speed.
//Between FIRE_DISTANCE and STOP_FIRE_DISTANCE.
	//Head at Target, fire max at Target if in fire cone, slow speed.
//Closer than STOP_FIRE_DISTANCE.
	//Head at Target, max speed.
	//If collide with Target, or longer than 2 seconds. retreat.
	private bool attemptedCollisionBusy = false;
	//Retreat runs uninterrupted until at RETREAT_DISTANCE.
	private bool retreatBusy = false;

//The Target diameter varies with its orientation, so this is approximate.
	private const float TARGET_RADIUS = 1f;

//Decide what to do.
	private void Decide()
	{
	//If retreating, continue it until complete.
		if (retreatBusy)
		{
			SetDesired(MAX_FORCE, -headingToTarget);
			if (distanceToTarget > RETREAT_DISTANCE)
				retreatBusy = false;
			return;
		}
	//Farther than FIRE_DISTANCE, head toward Target full speed.
		if (distanceToTarget > FIRE_DISTANCE)
		{
			SetDesired(MAX_FORCE, headingToTarget);
		}
	//Closer than STOP_FIRE_DISTANCE, head toward Target full speed, attempt to collide.
		else if (distanceToTarget < STOP_FIRE_DISTANCE)
		{
		//If a collision with Target, retreat.
			if (FS.TargetCollison)
			{
				attemptedCollisionBusy = false;
				retreatBusy = true;
				Invoke("StopRetreat", 5f);
				return;
			}
		//Trying our best to collide.
			if (attemptedCollisionBusy)
			{
				SetDesired(MAX_FORCE, headingToTarget);
			}
		//Start trying to collide abd set the timeout to retreat.
			else
			{
				attemptedCollisionBusy = true;
				Invoke("SetRetreat", 5f);
			}
		}
	//Between FIRE_DISTANCE and STOP_FIRE_DISTANCE, fire Missles.
		else
		{
			SetDesired(LOW_FORCE, headingToTarget);
			if (Vector3.Angle(headingToTarget, myForward) <= (Mathf.Atan2(TARGET_RADIUS, distanceToTarget) * Mathf.Rad2Deg))
				FireMissle();
		}
	}

//Timeout when attempting collision.
	private void SetRetreat()
	{
		retreatBusy = true;
	}

//Timeout when retreating.
	private void StopRetreat()
	{
		retreatBusy = false;
	}

//Set all desired values.
	private const float SMALL_ANGLE = 5f;
	private void SetDesired(float force, Vector3 heading)
	{
		desiredForce = force;
		desiredHeading = heading;
	//Set desiredTorque from desiredHeading.
		float angle = Vector3.SignedAngle(desiredHeading, myForward, Vector3.forward);
	//Use 0 when small angle, since high angular drag will slow it.
		if ((angle < SMALL_ANGLE) && (angle > -SMALL_ANGLE))
			desiredTorque = 0f;
	//Large angles, use 0 force to turn in place.
		else
		{
			desiredForce = MIN_FORCE;
			desiredTorque = MAX_TORQUE * Mathf.Sign(angle);
		}
	}

}
