//Copyright 2022 JerryLive.
//Handles a single Audio channel.

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class AudioChannelScript : MonoBehaviour
{

//This handles a channel including the sound and the mixer slider.
	[HideInInspector] public AudioSource sound;
	public AudioClip clip;
	public Slider slider;

//The sound can be played as a one-shot or repeating.
	private bool repeat;
//BPM is beats per minute.
	private float BPM;
	private int nFrames;
	private int frame;

//Pitch parameters.
	public float minPitch = 1f;
	public float maxPitch = 1f;


//Start here.##############################################################
	private void Awake()
	{
		sound = GetComponent<AudioSource>();
		sound.clip = clip;
		repeat = false;
		BPM = 120f;
		nFrames = (int)(50f * 60f / BPM);
	}

//Volume.
	public void OnValueChanged(float value)
	{
		sound.volume = value;
	}

//Play once.######################
	public void Play()
	{
		sound.pitch = Random.Range(minPitch, maxPitch);
		sound.PlayOneShot(clip);
		repeat = false;
	}
//This MUST have Clip loaded into AudioSource and set to looped (in the Inspector).
//It does not play the clip in this file.
	public void PlayLooped()
	{
		sound.pitch = Random.Range(minPitch, maxPitch);
		sound.Play();
		repeat = false;
	}
//Stop Playing.
	public void Stop()
	{
		sound.Stop();
	}

//Repeat.##########################
//Set repeat BPM.
//convert beats per minute to Physics frames per beat.
	public void SetRepeatBPM(float bpm)
	{
		BPM = bpm;
		nFrames = (int)(50f * 60f / BPM);
	}
//Start repeat Playing.
	public void StartRepeat()
	{
		repeat = true;
		frame = 0;
	}
//Stop repeat Play.
	public void StopRepeat()
	{
		repeat = false;
	}
//Every Physics Frame.
	private void FixedUpdate()
	{
		if (repeat)
		{
			frame += 1;
			if (frame >= nFrames)
			{
				frame = 0;
				sound.PlayOneShot(clip);
			}
		}
	}

}
