//Copyright 2022 JerryLive.
//Handles a single AudioParticle channel.

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class AudioParticleChannelScript : MonoBehaviour
{

//This handles a channel including:
//A single sound, the mixer slider, and up to 4 ParticleSystems.
//It will play everything that exists.
	[HideInInspector] public AudioSource sound;
	public AudioClip clip = null;
	public Slider slider;

//It will play a one-shot or repeating.
	private bool repeat;
//BPM is beats per minute.
	private float BPM;
	private int nFrames;
	private int frame;

//Pitch parameters.
	public float minPitch = 1f;
	public float maxPitch = 1f;

//ParticleSystems.
	public List<ParticleSystem> particles;


//Start here.##############################################################
	private void Awake()
	{
		sound = GetComponent<AudioSource>();
		sound.clip = clip;
		repeat = false;
		BPM = 120f;
		nFrames = (int)(50f * 60f / BPM);
	}

//Volume.
	public void OnValueChanged(float value)
	{
		sound.volume = value;
	}

//Play once.######################
	public void Play()
	{
		PlayAll();
		repeat = false;
	}
	//Move particles[0] to location and play once.
	public void PlayLocation(Vector3 location)
	{
		particles[0].transform.position = location;
		PlayAll();
		repeat = false;
	}
	//Stop Playing.
	public void Stop()
	{
	}

//Repeat.##########################
//Set repeat BPM.
//convert beats per minute to Physics frames per beat.
	public void SetRepeatBPM(float bpm)
	{
		BPM = bpm;
		nFrames = (int)(50f * 60f / BPM);
	}
//Start repeat Playing.
	public void StartRepeat()
	{
		repeat = true;
		frame = 0;
	}
//Stop repeat Play.
	public void StopRepeat()
	{
		repeat = false;
	}
//Every Physics Frame.
	private void FixedUpdate()
	{
		if (repeat)
		{
			frame += 1;
			if (frame >= nFrames)
			{
				frame = 0;
				PlayAll();
			}
		}
	}

//Play everything.
	private void PlayAll()
	{
		if (clip != null)
		{
			sound.pitch = Random.Range(minPitch, maxPitch);
			sound.PlayOneShot(clip);
		}
		for (int i = 0; i < particles.Count; i++)
		{
			particles[i].Play();
		}
	}

}
