//Copyright 2022 JerryLive
//Handles all Audio.

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class AudioScript : MonoBehaviour
{

//This handles all audio.

//There are several channels of Audio.
	public AudioChannelScript klaxonChannel;			//At start only.
	public AudioChannelScript beatChannel;				//Low Kick.
	public AudioChannelScript steamChannel;				//Steam burst at Catapult start.
	public AudioChannelScript engineChannel;			//Engine sound.
	public AudioChannelScript fireChannel;				//firing a missle
	public AudioChannelScript hitChannel;					//When missle hits target.


//SteamBea thas audio and particles.
	public AudioParticleChannelScript steamBeatChannel;		//Steam burst in time with Beat.


//Start here.##############################################################
	private void Awake()
	{
	}

//New Frame every 20ms.
//Play Ambient sound(s).
	private void FixedUpdate()
	{
		if (Global.DoNotRun)
			return;
	}

//Play Klaxon.
	public void PlayKlaxon()
	{
		klaxonChannel.Play();
	}

//Set Beat and SteamBeat.
	public void SetBeats(float BPM)
	{
		beatChannel.SetRepeatBPM(BPM);
		steamBeatChannel.SetRepeatBPM(BPM / 2f);
	}
	//Start Beat and SteamBeat.
	public void StartBeats()
	{
		beatChannel.StartRepeat();
		steamBeatChannel.StartRepeat();
	}
	//Stop Beat and SteamBeat.
	public void StopBeats()
	{
		beatChannel.StopRepeat();
		steamBeatChannel.StopRepeat();
	}

//Play Steam burst.
	public void PlaySteamBurst()
	{
		steamChannel.Play();
	}

//Play looped Engine.
	public void PlayEngine()
	{
		engineChannel.PlayLooped();
	}
//Stop Engine.
	public void StopEngine()
	{
		engineChannel.Stop();
	}

//Play missle fire.
	public void PlayFire()
	{
		fireChannel.Play();
	}

//Play missle hit and particle at location.
	public void PlayHit()
	{
		hitChannel.Play();
	}


//Persistant Items.##########################################
//These are called by GameMaster.
//Load settings.
	public void LoadSettings()
	{
		klaxonChannel.sound.volume = PlayerPrefs.GetFloat("KlaxonVolume", 0.7f);
			klaxonChannel.slider.value = klaxonChannel.sound.volume;
		beatChannel.sound.volume = PlayerPrefs.GetFloat("BeatVolume",0.7f);
			beatChannel.slider.value = beatChannel.sound.volume;
		steamBeatChannel.sound.volume = PlayerPrefs.GetFloat("SteamBeatVolume",0.7f);
			steamBeatChannel.slider.value = steamBeatChannel.sound.volume;
		steamChannel.sound.volume = PlayerPrefs.GetFloat("SteamVolume",0.7f);
			steamChannel.slider.value = steamChannel.sound.volume;
		engineChannel.sound.volume = PlayerPrefs.GetFloat("EngineVolume",0.7f);
			engineChannel.slider.value = engineChannel.sound.volume;
		fireChannel.sound.volume = PlayerPrefs.GetFloat("FireVolume",0.7f);
			fireChannel.slider.value = fireChannel.sound.volume;
		hitChannel.sound.volume = PlayerPrefs.GetFloat("HitVolume",0.7f);
			hitChannel.slider.value = hitChannel.sound.volume;
	}

//Save settings.
	public void SaveSettings()
	{
		PlayerPrefs.SetFloat("KlaxonVolume", klaxonChannel.sound.volume);
		PlayerPrefs.SetFloat("BeatVolume",beatChannel.sound.volume);
		PlayerPrefs.SetFloat("SteamBeatVolume",steamBeatChannel.sound.volume);
		PlayerPrefs.SetFloat("SteamVolume",steamChannel.sound.volume);
		PlayerPrefs.SetFloat("EngineVolume",engineChannel.sound.volume);
		PlayerPrefs.SetFloat("FireVolume",fireChannel.sound.volume);
		PlayerPrefs.SetFloat("HitVolume",hitChannel.sound.volume);
	}
}
