//Copyright 2022 JerryLive
//Handles all Flyers.

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class FlyerMasterScript : MonoBehaviour
{

//FlyerMaster sets the Player location and instantiates all AI_Flyers.
//During the Game it keeps track of all Flyers for the AI.
//If the Player Dies or is the last flying, it ends the Game.

//Flyers.
	private const int MAX_FLYERS = 4;
	public GameObject FlyerPrefab;			//only AI Flyers
//Each Flyer has a Catapult Carriage.
	public GameObject CarriagePrefab;
//There are 4 different color schemes.
	//Each Flyer has 7 separate Sprites, each with its own color.
	//Engine and BodyRear are the same color for all Flyers.
	//Tail, WingTip_L and WingTip_R are TailColor.
	//Wing_L and Wing_R are WingColor.
	private List<Color> TailColor = new List<Color>
	{
		new Color(1.00f, 0.94f, 0.65f, 1.00f),
		new Color(0.88f, 1.00f, 0.58f, 1.00f),
		new Color(0.50f, 0.81f, 1.00f, 1.00f),
		new Color(1.00f, 0.53f, 0.91f, 1.00f)
	};
	private List<Color> WingColor = new List<Color>
	{
		new Color(1.00f, 0.87f, 0.20f, 1.00f),
		new Color(0.58f, 0.81f, 0.00f, 1.00f),
		new Color(0.35f, 0.56f, 0.70f, 1.00f),
		new Color(0.83f, 0.19f, 0.71f, 1.00f)
	};

//There are 4 locations where Flyers can be placed.
	private List<Vector3> FlyerPosition = new List<Vector3>
	{
		new Vector3(-8.56f, -4.81f, -0.01f),
		new Vector3(-8.56f,  4.81f, -0.01f),
		new Vector3( 8.56f,  4.81f, -0.01f),
		new Vector3( 8.56f, -4.81f, -0.01f)
	};
	private List<Vector3> FlyerRotation = new List<Vector3>
	{
		new Vector3(0f, 0f, 29.36f),
		new Vector3(0f, 0f, -29.36f),
		new Vector3(0f, 0f, -150.64f),
		new Vector3(0f, 0f, 150.64f)
	};
//The center of the arena.
	public GameObject center;
//Highlight the Player at start.

//References.
	private GameObject Player;
	private GameMasterScript GMS;
	public GameObject Missle;
	public GameObject Hit;


//Start here.#######################################################
	private void Awake()
	{
		Player = GameObject.FindGameObjectWithTag("Player");
		GMS = FindObjectOfType<GameMasterScript>();
		Global.CreateMissleHitPool(Missle, Hit);
	}

//This is called by GameMaster to setup the Game.
	public void SetupGame()
	{
		switch(Global.gameType)
		{
			case Global.enumGameType.MATCH:
				Global.nFlyers = Global.nMATCH;
				break;
			case Global.enumGameType.MELEE:
				Global.nFlyers = Global.nMELEE;
				break;
		}
		int i, nPosition, nColor;
		Quaternion rotation;
		Vector3 position;
		for (i = 0; i < Global.nFlyers; i++)
		{
			nPosition = Random.Range(0, FlyerPosition.Count);
			nColor = Random.Range(0, TailColor.Count);
			position = FlyerPosition[nPosition];
			rotation = Quaternion.Euler(FlyerRotation[nPosition]);
		//Player Flyer.
			if (i == 0)
			{
				Global.Flyer[0].GO = Player;
				Global.Flyer[0].GO.transform.position = position;
				Global.Flyer[0].GO.transform.rotation = rotation;
			}
		//AI Flyers.
			else
			{
				Global.Flyer[i].GO = Instantiate(FlyerPrefab, position, rotation);
				Global.Flyer[i].targetIndex = 0;
			}
		//All Flyers.
			Global.Flyer[i].alive = true;
			Global.Flyer[i].weakEngine = false;
			Global.Flyer[i].RB2 = Global.Flyer[i].GO.GetComponent<Rigidbody2D>();
			Global.Flyer[i].FS = Global.Flyer[i].GO.GetComponent<FlyerScript>();
			Global.Flyer[i].FS.myIndex = i;
			Global.Flyer[i].Carriage = Instantiate(CarriagePrefab, position, rotation);
			Global.CreatePart(i);
			Global.Flyer[i].FS.LoadPart(WingColor[nColor], TailColor[nColor]);
			FlyerPosition.RemoveAt(nPosition);
			FlyerRotation.RemoveAt(nPosition);
			TailColor.RemoveAt(nColor);
			WingColor.RemoveAt(nColor);
		}
	//All Flyers present and accounted for, but they need Targets.
		PickTargets();
	}

//Targets.
//AI Flyers all have to have a Target to Attack.
//The Player doesn't get a Target.
//If a Target dies, all Targets are redone.
	private List<int> targets = new List<int>();

	private void PickTargets()
	{
		int i, n;
	//Make a List of available Targets.
		for (i = 0; i < Global.nFlyers; i++)
		{
			if (Global.Flyer[i].alive)
				targets.Add(i);
		}
	//Add some extra Player Targets.
		targets.Add(0);
		targets.Add(0);
	//Pick targets at random.
		for (i = 1; i < Global.nFlyers; i++)
		{
			if (Global.Flyer[i].alive)
			{
				n = i;
				while(n == i)
				{
					n = Random.Range(0, targets.Count);
					if (targets[n] != i)
					{
						Global.Flyer[i].targetIndex = targets[n];
						targets.RemoveAt(n);
					}
				}
			}
		}
	}


//Every Physics Frame (50Hz).########################################
	private void FixedUpdate()
	{
		if (Global.DoNotRun)
			return;
		if (CheckGameOver())
			return;
	//Find out where all the Flyers are.
		Global.CalculateWhere();
	//If any Targets are dead, assign new Targets.
		CheckTargets();
	}

//Have any Targets died?
	private void CheckTargets()
	{
		for (int i = 1; i < Global.nFlyers; i++)
		{
			if (Global.Flyer[i].alive)
			{
				if (! Global.Flyer[Global.Flyer[i].targetIndex].alive)
				{
					PickTargets();
					return;
				}
			}		
		}
	}

//If the Player dies, or all other Flyers are dead, the game is over.
	private bool CheckGameOver()
	{
	//The Player is dead...
		if (! Global.Flyer[0].alive)
			GMS.EndGame(false);
	//Or maybe the Player is the last flying.
		else
		{
			for (int i = 1; i < Global.nFlyers; i++)
			{
				if (Global.Flyer[i].alive)
					return false;
			}
			GMS.EndGame(true);
		}
		Global.DoNotRun = true;
		return true;
	}
}
