//Copyright 2022 JerryLive
//Handles a Flyer.

using System.Text;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class FlyerScript : MonoBehaviour
{

//This script MUST be on every Flyer, including the Player.
//It handles movement, collisions and apprearance.
//This is the index into Global.Flyer[], Player is always 0.
	[HideInInspector] public int myIndex;

//Movement.#############################
	//Flyers are entirely moved by Physics.
	//They can move forward and rotate endlessly.
	//Movement input can come either from the Player or AI.
	//The Player: AD keys rotate Left/Right, W key moves Forward.
	//The AI: does what it wants.
		[HideInInspector] public float FlyerTorque;		//-1f to 1f
		private float torque = 0f;
		private const float TORQUE_SF = -1.2f;				//affects every Flyer
		[HideInInspector] public float FlyerForce;		//0f to 1f
		private float force = 0f;
		private const float FORCE_SF = 3f;						//affects every Flyer

//Colors.################################
	//Each Flyer has 7 Sprites, each with a SpriteRenderer.
	//The Engine and BodyRear Sprite colors are the same for all Flyers.
	//Wing_L and Wing_R are the same color.
	//Tail, WingTip_L and WingTip_R are the same color.
	//This is handled and described in FlyerMasterScript.
	public GameObject Engine;
	public GameObject BodyRear;
	public GameObject Tail;
	public GameObject Wing_L;
	public GameObject Wing_R;
	public GameObject WingTip_L;
	public GameObject WingTip_R;

//Missles.###############################
//All Flyers fire Missles, which are pooled in Global.
	public GameObject MissleFire;						//locate just beyond the Flyer nose
	private const float MISSLE_FORCE_X = 5f;

//Audio.#################################
//Flyers play a looped motor sound and a fire sound when firing a missle.
	private AudioScript AS;


//Start here.###################################################################
	private void Awake()
	{
		AS = FindObjectOfType<AudioScript>();
	}

//Load the GameObjects, SpriteRenderers and Colors into Part.
//Engine and BodyRear are colored in the Sprite.
	public void LoadPart(Color WingColor, Color TailColor)
	{
		Global.Flyer[myIndex].Part[Global.ENGINE].GO = Engine;
		Global.Flyer[myIndex].Part[Global.ENGINE].SR = Engine.GetComponent<SpriteRenderer>();
		Global.Flyer[myIndex].Part[Global.BODY_REAR].GO = BodyRear;
		Global.Flyer[myIndex].Part[Global.BODY_REAR].SR = BodyRear.GetComponent<SpriteRenderer>();
		Global.Flyer[myIndex].Part[Global.TAIL].GO = Tail;
		Global.Flyer[myIndex].Part[Global.TAIL].SR = Tail.GetComponent<SpriteRenderer>();
		Global.Flyer[myIndex].Part[Global.TAIL].SR.color = TailColor;
		Global.Flyer[myIndex].Part[Global.WING_L].GO = Wing_L;
		Global.Flyer[myIndex].Part[Global.WING_L].SR = Wing_L.GetComponent<SpriteRenderer>();
		Global.Flyer[myIndex].Part[Global.WING_L].SR.color = WingColor;
		Global.Flyer[myIndex].Part[Global.WING_R].GO = Wing_R;
		Global.Flyer[myIndex].Part[Global.WING_R].SR = Wing_R.GetComponent<SpriteRenderer>();
		Global.Flyer[myIndex].Part[Global.WING_R].SR.color = WingColor;
		Global.Flyer[myIndex].Part[Global.WING_TIP_L].GO = WingTip_L;
		Global.Flyer[myIndex].Part[Global.WING_TIP_L].SR = WingTip_L.GetComponent<SpriteRenderer>();
		Global.Flyer[myIndex].Part[Global.WING_TIP_L].SR.color = TailColor;
		Global.Flyer[myIndex].Part[Global.WING_TIP_R].GO = WingTip_R;
		Global.Flyer[myIndex].Part[Global.WING_TIP_R].SR = WingTip_R.GetComponent<SpriteRenderer>();
		Global.Flyer[myIndex].Part[Global.WING_TIP_R].SR.color = TailColor;
	}

//Movement.
//Every Physics Frame.
	private void FixedUpdate()
	{
		if (Global.DoNotRun)
			return;
		CheckParts();
	//Rotate.
		torque = FlyerTorque * TORQUE_SF;
		Global.Flyer[myIndex].RB2.AddTorque(torque);
	//Translate.
		force = FlyerForce * FORCE_SF;
		Global.Flyer[myIndex].RB2.AddForce(transform.right * force);
	}

//Missle.
	public void FireMissle()
	{
		GameObject Missle = Global.GetMissle(MissleFire.transform.position);
		MissleScript MS = Missle.GetComponent<MissleScript>();
		MS.SetStartPosition(MissleFire.transform.position);
		Rigidbody2D MRB2 = Missle.GetComponent<Rigidbody2D>();
		MRB2.AddForce(MISSLE_FORCE_X * transform.right, ForceMode2D.Impulse);
		AS.PlayFire();
	}


//Collisions.###################################################################
//Collisions reduce the transparency of parts.
//When they get too low, the part is set inActive.
	private const float INACTIVE_TRANSPARENCY = 0.1f;
//This affects the performance of the Flyer.

//When a collision happens with:
	//The Cage, OnCollisionEnter2D() is called with otherName = Cage.
	//A Missle, OnCollisionEnter2D() with otherName = Missle.
	//Another Flyer, OnCollisionEnter2D() with myName = this Flyer, otherName = other Flyer.

//There can be multiple contacts at once.
//However, in testing, the only time this happens is 2 identical contacts.
//Only use the firstcontact.
	private ContactPoint2D[] contactsEnter = new ContactPoint2D[16];

//The index into Global.Flyer[] of the Flyer collided with.
	private int otherIndex;
//Set true if otherIndex is the Target, set false if not.
	[HideInInspector] public bool TargetCollison;

//Damage:
	//Damage depends on the part hit, what it hit, and the relativeVelocity.
	//Hitting the Cage is not a big deal.
		//Damage is relativeVelocity * partWeight * DAMAGE_CAGE.
		private const float DAMAGE_CAGE = 0.1f;
	//Hitting a Missle is bad news.
		//Damage is relativeVelocity * partWeight * DAMAGE_MISSLE.
		private const float DAMAGE_MISSLE = 0.05f;
	//Hitting another Flyer depends on what hits what.
		//An Engine hitting a Tail does little damage to the Engine, but a lot to the Tail.
		//Damage is relativeVelocity * (other.partWeight / my.partWeight) * DAMAGE_FLYER.
		private const float DAMAGE_FLYER = 0.1f;


//All collisions come here.#################################
	private void OnCollisionEnter2D(Collision2D collision)
	{
		if (Global.DoNotRun)
			return;
		StringBuilder SB = new StringBuilder();
		int n, iPart, iOtherPart;
		Color partColor;
		float damage = 0;
		n = collision.GetContacts(contactsEnter);
	//Use the first contact only.
		ContactPoint2D contact = contactsEnter[0];
		float relativeVelocity = contact.relativeVelocity.magnitude;
	//For whatever reason, collider and otherCollider are reversed.
		string myName =	contact.otherCollider.gameObject.name;
		string otherName =	contact.collider.gameObject.name;
	//Get the collided part.
  		for (iPart = 0; iPart < Global.nFLYER_PARTS; iPart++)
		{
			if (myName == Global.Flyer[myIndex].Part[iPart].name)
				break;
		}
	//Collision with the Cage.
		if (otherName == "Cage")
			damage = relativeVelocity * Global.Flyer[myIndex].Part[iPart].damageWeight * DAMAGE_CAGE;
	//Missle collision.
 		else if (otherName == "Missle(Clone)")
			damage = relativeVelocity * Global.Flyer[myIndex].Part[iPart].damageWeight * DAMAGE_MISSLE;
	//Must be another Flyer.
		else
		{
		//Get the Global.Flyer[] index of collided Flyer.
			GameObject otherGO = contact.collider.attachedRigidbody.gameObject;
			for (otherIndex = 0; otherIndex < Global.nFlyers; otherIndex++)
			{
				if (Global.Flyer[otherIndex].GO == otherGO)
					break;
			}
			//Set TargetCollison.
			if (Global.Flyer[myIndex].targetIndex == otherIndex)
				TargetCollison = true;
			else
				TargetCollison = false;
		//Get the collided part.
			for (iOtherPart = 0; iOtherPart < Global.nFLYER_PARTS; iOtherPart++)
			{
				if (otherName == Global.Flyer[myIndex].Part[iOtherPart].name)
					break;
			}
			damage = relativeVelocity * (Global.Flyer[myIndex].Part[iOtherPart].damageWeight
							/ Global.Flyer[myIndex].Part[iPart].damageWeight) * DAMAGE_FLYER;
		}
	//Damage affects the transparency of the part.
		partColor = Global.Flyer[myIndex].Part[iPart].SR.color;
		partColor.a -= damage;
		Global.Flyer[myIndex].Part[iPart].SR.color = partColor;
	//Too little opacity leads to part death.
		if (partColor.a < INACTIVE_TRANSPARENCY)
		{
			Global.Flyer[myIndex].Part[iPart].GO.SetActive(false);
   		Global.Flyer[myIndex].Part[iPart].alive = false;
			return;
		}
	//If the engine is below 50%, it's weak.
		if (Global.Flyer[myIndex].Part[Global.ENGINE].SR.color.a < 0.5f)
			Global.Flyer[myIndex].weakEngine = true;
	//Print the collison for debug.
		//SB.Clear();
		//SB.Append("F" + myIndex.ToString() + " " + myName);
		//if (otherName == "Cage")
		//	SB.Append(" * Cage");
		//else if (otherName == "Missle(Clone)")
		//	SB.Append(" * Missle");
		//else
		//	SB.Append(" * F" + otherIndex.ToString() + " " + otherName);
		//SB.Append(" V=" + relativeVelocity.ToString("F2"));
		//SB.Append(" X=" + damage.ToString("F2"));
		//print(SB.ToString());
	}

//Check for dead parts.
//If necessary, modify FlyerTorque or FlyerForce, or Kill this Flyer.
	private void CheckParts()
	{
	//A dead engine or inner wing kills the Flyer.
		if ((! Global.Flyer[myIndex].Part[Global.ENGINE].alive) ||
				(! Global.Flyer[myIndex].Part[Global.WING_L].alive) ||
				(! Global.Flyer[myIndex].Part[Global.WING_R].alive))
		{
			Global.Flyer[myIndex].alive = false;
			Global.Flyer[myIndex].GO.SetActive(false);
			return;
		}
	//A dead rear body kills the tail.
		if (! Global.Flyer[myIndex].Part[Global.BODY_REAR].alive)
			Global.Flyer[myIndex].Part[Global.TAIL].alive = false;
	//Without a Tail, randomly rotate left and right.
		if (! Global.Flyer[myIndex].Part[Global.TAIL].alive)
			FlyerTorque += Random.Range(-0.1f, 0.1f);
	//Without a wing tip, turn away.
		if (! Global.Flyer[myIndex].Part[Global.WING_TIP_L].alive)
			FlyerTorque += 0.1f;
		if (!Global.Flyer[myIndex].Part[Global.WING_TIP_R].alive)
			FlyerTorque -= 0.1f;
	//Without both wing tips, slow down.
		if ((! Global.Flyer[myIndex].Part[Global.WING_TIP_L].alive) && (! Global.Flyer[myIndex].Part[Global.WING_TIP_R].alive))
			FlyerForce *= 0.7f;
	//If the engine is weak, slow down.
		if (Global.Flyer[myIndex].weakEngine)
			FlyerForce *= 0.7f;
	}

}
