//Copyright 2022 JerryLive
//The Game starts and ends here.

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.EventSystems;

public class GameMasterScript : MonoBehaviour
{

//Game Type.
	public Toggle matchToggle;
	public Toggle meleeToggle;

//Difficulty.
	public Toggle hardToggle;

//Force end with escape.
	[HideInInspector] public static bool escapeEnd = false;

//The Start Canvas.
	public GameObject startCanvas;
//End Canvases.
	public GameObject whoopsCanvas;
	public GameObject winCanvas;
	public GameObject deathCanvas;
//Mixer Canvas.
	public GameObject audioCanvas;

//The Start Light.
	public GameObject redLight;
	public GameObject yellowLight;
	public GameObject greenLight;

//Catapult.
	public PlayerCatapultScript PCS;

//Start Button.
	public GameObject startButton;
//Pause Button.
	public Image pauseButton;
//Mixer Button.
	public Image volumeButton;

//The Beat starts when the StartCanvas is removed.
//The steamBeat is puffing in half time with the Beat.
//Beats (Beat and steamBeat) continue until the end of the game.
	private AudioScript AS;
	private float BPM;

//References.
	private FlyerMasterScript FMS;


//Start here.################################################
//The Start Canvas is displayed and then wait for the Start Button pressed.
	void Start()
	{
		AS = FindObjectOfType<AudioScript>();
		FMS = FindObjectOfType<FlyerMasterScript>();
		LoadSettings();
		startCanvas.SetActive(true);
		EventSystem.current.SetSelectedGameObject(startButton);

	}

//Check for escape.
	void Update()
	{
		if (Input.GetKey(KeyCode.Escape))
		{
			escapeEnd = true;
			WhoopsGame();
		}
	}

//Start game.
	public void StartGame()
	{
	//Game Type.
		if (matchToggle.isOn)
			Global.gameType = Global.enumGameType.MATCH;
		else
			Global.gameType = Global.enumGameType.MELEE;
	//Difficulty.
		if (hardToggle.isOn)
			Global.AI_FramesPerMissle = Global.HARD_FRAMES_PER_MISSLE;
		else
			Global.AI_FramesPerMissle = Global.NORMAL_FRAMES_PER_MISSLE;
	//Instantiate all AI Flyers and get ready to Rumble...
		FMS.SetupGame();
		startCanvas.SetActive(false);
		redLight.SetActive(true);
		AS.PlayKlaxon();
		InvokeRepeating("StartLight", 2f, 2f);
		PCS.SetupCatapult();
		BPM = 120f;
		AS.SetBeats(BPM);
		AS.StartBeats();
	}

//Start Light.
	public void StartLight()
	{
		if(redLight.activeSelf)
		{
			redLight.SetActive(false);
			yellowLight.SetActive(true);
			AS.PlayKlaxon();
		}
	//Start the Catapults.
		else if(yellowLight.activeSelf)
		{
			yellowLight.SetActive(false);
			greenLight.SetActive(true);
			AS.PlayKlaxon();
			CancelInvoke();
			PCS.StartCatapult();
			CatapultScript CS;
			for (int i = 1; i < Global.nFlyers; i++)
			{
				CS = Global.Flyer[i].GO.GetComponent<CatapultScript>();
				CS.StartCatapult();
			}
		//The Boilers emit a large, noisy blast of Steam.
			AS.PlaySteamBurst();
		//Start your engines.
			AS.PlayEngine();
			AS.PlayEngine();
		//Little more encouragement.
			BPM = 140f;
			AS.SetBeats(BPM);
		}
	}

//Toggle pause.
	public void TogglePause()
	{
		if(Global.DoNotRun)
			return;
		if(volumeButton.color == Color.red)
			return;
		if(Time.timeScale == 0f)
		{
			pauseButton.color = Color.gray;
			Time.timeScale = 1f;
		}
		else
		{
			pauseButton.color = Color.red;
			Time.timeScale = 0f;
		}
	}

//Toggle mixer visibility.
	public void ToggleMixer()
	{
		if(Global.DoNotRun)
			return;
		if (pauseButton.color == Color.red)
			return;
		if(audioCanvas.activeInHierarchy)
		{
			audioCanvas.SetActive(false);
			volumeButton.color = Color.gray;
			AS.PlayEngine();
			Time.timeScale = 1f;
		}
		else
		{
			audioCanvas.SetActive(true);
			volumeButton.color = Color.red;
			AS.StopEngine();
			Time.timeScale = 0f;
		}
}


//End game.#######################################################
//Game has been ended with esc key.
	public void WhoopsGame()
	{
		AS.StopEngine();
		Global.DoNotRun = true;
		whoopsCanvas.SetActive(true);
		Invoke("AllDone", 3f);
	}

//Game has been ended with a win or death.
	public void EndGame(bool win)
	{
		AS.StopEngine();
		Global.DoNotRun = true;
		if (win)
			winCanvas.SetActive(true);
		else
			deathCanvas.SetActive(true);
		Invoke("AllDone", 5f);
	}

//Really End Game after delay.
	public void AllDone()
	{
		SaveSettings();
		#if UNITY_EDITOR
			UnityEditor.EditorApplication.isPlaying = false;
		#endif
		Application.Quit();
	}


//Persistant Items.##########################################
//Load settings.
	private void LoadSettings()
	{
		AS.LoadSettings();
	}

//Save settings.
	private void SaveSettings()
	{
		AS.SaveSettings();
	}

}
