//Copyright 2022 JerryLive
//Everything here is available to all Scripts.

//This Project was written with:  Indenting=none, Tab size=2, Keep Tabs.
//Set this in the VS menu: Tools - Options - Text Editor - C# - Tabs

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Global : MonoBehaviour
{

//Do not let any Updates or FixedUpdates run until start button is pressed.
	public static bool DoNotRun = true;

//The Game is MATCH=2 or MELEE=4 Flyers, including the Player.
	public enum enumGameType { MATCH, MELEE };
	public static enumGameType gameType;
	public const int nMATCH = 2;
	public const int nMELEE = 4;

//Difficulty applies to AI Flyers only.
//Tjis is the number of physics frames before another missle can be fired.
	public const int NORMAL_FRAMES_PER_MISSLE = 40;
	public const int HARD_FRAMES_PER_MISSLE = 12;
	public static int AI_FramesPerMissle;

//Missles are fired by Flyers and die after 
	public const float MISSLE_KILL_DISTANCE = 3f;

//The force applied on the Catapult.
	public const float CATAPULT_FORCE = 2f;


//Flyers.#######################################################################
	public static int nFlyers;
//Flyers are instantiated in FlyerMasterScript.
//Flyer[0] is always the Player.
	public struct strucFlyer
	{
		public bool alive;
		public bool weakEngine;
		public GameObject GO;
		public Rigidbody2D RB2;
		public FlyerScript FS;
		public GameObject Carriage;
		public strucFlyerPart[] Part;
		public int targetIndex;						//AI Flyers only
	};
	public static strucFlyer[] Flyer = new strucFlyer[nMELEE];

//A Flyer is 7 Sprite GameObjects, each with a SpriteRenderer and a PolygonCollider2D.
	public const int nFLYER_PARTS = 7;
		public const int ENGINE = 0;			//always Red
		public const int BODY_REAR = 1;		//always Orange
	//These colors are in FlyerMasterScript.
		public const int TAIL = 2;				//TailColor
		public const int WING_L = 3;			//WingColor
		public const int WING_R = 4;			//WingColor
		public const int WING_TIP_L = 5;	//TailColor
		public const int WING_TIP_R = 6;	//TailColor
	public struct strucFlyerPart
	{
		public bool alive;
		public string name;
		public float damageWeight;
		public GameObject GO;
		public SpriteRenderer SR;
	};

//Create the starting Part arrays.
	public static void CreatePart(int i)
	{
		Flyer[i].Part = new strucFlyerPart[nFLYER_PARTS];
		Flyer[i].Part[ENGINE].alive = true;
		Flyer[i].Part[ENGINE].name = "Engine";
		Flyer[i].Part[ENGINE].damageWeight = 0.8f;
		Flyer[i].Part[BODY_REAR].alive = true;
		Flyer[i].Part[BODY_REAR].name = "BodyRear";
		Flyer[i].Part[BODY_REAR].damageWeight = 0.5f;
		Flyer[i].Part[TAIL].alive = true;
		Flyer[i].Part[TAIL].name = "Tail";
		Flyer[i].Part[TAIL].damageWeight = 0.4f;
		Flyer[i].Part[WING_L].alive = true;
		Flyer[i].Part[WING_L].name = "Wing_L";
		Flyer[i].Part[WING_L].damageWeight = 0.5f;
		Flyer[i].Part[WING_R].alive = true;
		Flyer[i].Part[WING_R].name = "Wing_R";
		Flyer[i].Part[WING_R].damageWeight = 0.5f;
		Flyer[i].Part[WING_TIP_L].alive = true;
		Flyer[i].Part[WING_TIP_L].name = "WingTip_L";
		Flyer[i].Part[WING_TIP_L].damageWeight = 0.3f;
		Flyer[i].Part[WING_TIP_R].alive = true;
		Flyer[i].Part[WING_TIP_R].name = "WingTip_R";
		Flyer[i].Part[WING_TIP_R].damageWeight = 0.3f;
	}


//Flyer Location.###############################################################
//Where each Flyer is in relation to all other Flyers.
	public struct strucWhere
	{
		public float distance;
		public Vector3 heading;
	};
	public static strucWhere[,] Where = new strucWhere[nMELEE, nMELEE];

//Calculate distance and heading between all Flyers.
	public static void CalculateWhere()
	{
		int n = -1;
		for (int i = 0; i < nFlyers; i++)
		{
			n += 1;
			if (! Flyer[i].alive)
				continue;
			for (int j = n; j < nFlyers; j++)
			{
				if (i == j)
					Where[i,j].distance = 0f;
				else
				{
					Where[i,j].distance = Vector3.Distance(Flyer[i].GO.transform.position, Flyer[j].GO.transform.position);
					Where[i,j].heading = (Flyer[j].GO.transform.position - Flyer[i].GO.transform.position).normalized;
					Where[j,i].distance = Where[i,j].distance;
					Where[j,i].heading = -Where[i,j].heading;
				}
			}
		}
	}


//Missles.######################################################################
//Rather than use Instantiate and destroy for Missles and Hit Particles, use pooling.
	public static List<GameObject> Missles;
	public static List<GameObject> Hits;
	public const int MAX_MISSLES_HITS = 32;

//Use Missles[].SetActive(false) or Hits[].SetActive(false) to return it to the Pool.

//Create the Pool with MAX_MISSLES.
	public static void CreateMissleHitPool(GameObject missle, GameObject hit)
	{
		GameObject temp;
		Missles = new List<GameObject>();
		Hits = new List<GameObject>();
		for (int i = 0; i < MAX_MISSLES_HITS; i++)
		{
			temp = Instantiate(missle);
			temp.SetActive(false);
			Missles.Add(temp);
			temp = Instantiate(hit);
			temp.SetActive(false);
			Hits.Add(temp);
		}
	}

//Return a Missle at location.
	public static GameObject GetMissle(Vector3 location)
	{
		for (int i = 0; i < MAX_MISSLES_HITS; i++)
		{
			if (! Missles[i].activeInHierarchy)
			{
				Missles[i].transform.position = location;
				Missles[i].transform.rotation = Quaternion.identity;
				Missles[i].SetActive(true);
				return Missles[i];
			}
		}
		return null;
	}

//Return a Hit at location.
	public static GameObject GetHit(Vector3 location)
	{
		for (int i = 0; i < MAX_MISSLES_HITS; i++)
		{
			if (! Hits[i].activeInHierarchy)
			{
				Hits[i].transform.position = location;
				Hits[i].transform.rotation = Quaternion.identity;
				Hits[i].SetActive(true);
				return Hits[i];
			}
		}
		return null;
	}
	
}
