//Copyright 2022 JerryLive
//Handles the Player Catapult, which controls all other Catapults.

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PlayerCatapultScript : MonoBehaviour
{

//The Player Catapult is the same as the AI Catapult except for...
	//Disables and enables the CageCollider (it blocks the Catapults).
	//Enables AI and Player Controls when the Flyers are clear of the CageCollider.

//The Flyer is located at the start point and headed down the Catapult.
	private Vector3 ZeroPos;

//When GameMaster removes the StartCanvas:
	//Disable the Cage Collider.
	public EdgeCollider2D CageCollider;
//At the green light:
	//The Carriage and Flyer accelerate.
	private bool catapultBusy;
//When the Carriage reaches the end:
	private const float STOP_DISTANCE = 1.8f;
	//The Carriage stops.
	//The Flyer continues at minimum speed.
//When the Flyer is clear of the Cage:
	private const float CLEAR_DISTANCE = 3f;
	//The Cage Collider is enabled.
	//The center light is disabled.
	public GameObject Center;
	//AI and Player are put in control.

//References.
	public GameObject HiLite;


//Start here.################################################
	private void Awake()
	{
		catapultBusy = false;
	}

//GameMaster has removed the Start Canvas.
	public void SetupCatapult()
	{
		CageCollider.enabled = false;
	}

//GameMaster has set the Green Light.
	public void StartCatapult()
	{
		ZeroPos = transform.position;
		catapultBusy = true;
	}

//Every Physics Frame.
	private void FixedUpdate()
	{
		if (! catapultBusy)
			return;
	//The Flyer and Carriage are catapulting.
		float distance = Vector3.Distance(ZeroPos, transform.position);
		if (distance < STOP_DISTANCE)
		{
			Global.Flyer[0].RB2.AddRelativeForce(new Vector3(Global.CATAPULT_FORCE, 0f, 0f));
			Global.Flyer[0].Carriage.transform.position = Global.Flyer[0].GO.transform.position;
		}
	//Reached the Catapult end.
	//Flyer just keeps going, Carriage stops.
		else if (distance < CLEAR_DISTANCE)
		{
			HiLite.SetActive(false);
			Global.Flyer[0].RB2.AddRelativeForce(new Vector3(Global.CATAPULT_FORCE, 0f, 0f));
		}
	//Clear of CageCollider.
	//Catapult is all done, enable PLayer and AI to control the Flyers.
		else
		{
			CageCollider.enabled = true;
			Center.SetActive(false);
			catapultBusy = false;
			Global.DoNotRun = false;
		}
	}

}
